/*
 * DataStore.java
 * The RecordStore wrapper
 * (c) Arcanum
 * http://faqsite.narod.ru
 */

import javax.microedition.rms.*;
import java.io.*;

public class DataStore {

//The store's name
    static final String STR_STORE   = "Renju";    
    RecordStore store;

    public DataStore() {

    }
 
 // Call it to store settings
     public void saveSettings(SetData data) {
 //  Byte is native for record store. So make a byte stream.
        ByteArrayOutputStream   setByteStream = new ByteArrayOutputStream();
 // Underlying data stream imlements writing the useful types of data
        DataOutputStream        setOutStream  = new DataOutputStream(setByteStream);
 // Simply put data to the stream 
        try {
            setOutStream.writeInt(data.time);
            setOutStream.writeBoolean(data.sound);
            setOutStream.writeBoolean(data.light);
        } catch(IOException e) {
            System.out.println("Excpt write:" + e);
        }
        byte setData[] = setByteStream.toByteArray();
        openStore();
        saveData(1, setData);
        closeStore();
    }
 // Call it to load settings    
    public SetData loadSettings()
    {        
        openStore();
        DataInputStream setInpStream = new DataInputStream(new ByteArrayInputStream(loadData(1)));
        try {
        SetData data = new SetData(
            setInpStream.readInt(),
            setInpStream.readBoolean(),
            setInpStream.readBoolean());
            closeStore();
            return data;        
        } catch(IOException e) {
            closeStore();            
            System.out.println("Excpt:" + e);
        }
        return null;
    }
// Saving BoardData structure. It's easy by using a byte stream
    public void saveBoard(BoardData data) {
        ByteArrayOutputStream   boardByteStream = new ByteArrayOutputStream();
        DataOutputStream        boardOutStream  = new DataOutputStream(boardByteStream);
        try {
            boardOutStream.writeInt(data.move);
            boardOutStream.writeInt(data.bsTime);
            boardOutStream.writeInt(data.wsTime);
            for(int i = 0; i < BoardData.BOARD_SIZE; i++) {
                for(int j = 0; j < BoardData.BOARD_SIZE; j++) {
                    boardOutStream.writeShort(data.board[i][j]);
                }
            }
            
        } catch(IOException e) {
            System.out.println("Excpt write:" + e);
        }
        byte boardData[] = boardByteStream.toByteArray();
        openStore();
        saveData(2, boardData);
        closeStore();  
    }
// Load BoardData structure. It's simple by using a stream
    public BoardData loadBoard() {
        openStore();
        DataInputStream boardInpStream = new DataInputStream(new ByteArrayInputStream(loadData(2)));
        try {
            int move        = boardInpStream.readInt();
            int bsTime      = boardInpStream.readInt();
            int wsTime      = boardInpStream.readInt();          
            int board[][]   = new int[BoardData.BOARD_SIZE][BoardData.BOARD_SIZE];
            for(int i = 0; i < BoardData.BOARD_SIZE; i++) {
                for(int j = 0; j < BoardData.BOARD_SIZE; j++) {
                    board[i][j] = (int)boardInpStream.readShort();
                }
            }
            closeStore();            
            BoardData data = new BoardData(board, move, bsTime, wsTime);
            return data;        
        } catch(IOException e) {
            closeStore();            
            System.out.println("Excpt:" + e);
        }
        return null;
    }
    
// opens the store or creates it    
    private void openStore() {
        try { 
            try { 
                store = RecordStore.openRecordStore(STR_STORE, false);
            } catch (RecordStoreNotFoundException e) { 
                store = RecordStore.openRecordStore(STR_STORE, true);
                initData();
            }
        } catch(Exception e) { 
            System.out.println("Excpt: openRecordStore()" + e);
        }
    }
// closes the store after the usage    
    private void closeStore() {
        if (store != null) { 
            try { 
                store.closeRecordStore();
            } catch (Exception e) {
                System.out.println("Excpt: closeRecordStore()" + e);
            }
        }
    }
// If storage just created we have to make the record with default data    
    private void initData() {
        {
            SetData setData                         = new SetData(300, true, true);
            ByteArrayOutputStream   setByteStream   = new ByteArrayOutputStream();
            DataOutputStream        setOutStream    = new DataOutputStream(setByteStream);
            try {
                setOutStream.writeInt(setData.time);
                setOutStream.writeBoolean(setData.sound);
                setOutStream.writeBoolean(setData.light);
            } catch(IOException e) {
                System.out.println("Excpt write:" + e);
            }
            byte data[] = setByteStream.toByteArray(); 
            
            try {
                store.addRecord(data, 0, data.length);
            } catch (Exception e) {
                System.out.println("Excpt: addRecord()" + e);
            }
        }
        {
            BoardData boardData                     = new BoardData(new int[BoardData.BOARD_SIZE][BoardData.BOARD_SIZE], 0, 0, 0);
            ByteArrayOutputStream   boardByteStream = new ByteArrayOutputStream();
            DataOutputStream        boardOutStream  = new DataOutputStream(boardByteStream);
            try {
                boardOutStream.writeInt(boardData.move);
                boardOutStream.writeInt(boardData.bsTime);
                boardOutStream.writeInt(boardData.wsTime);
                for(int i = 0; i < BoardData.BOARD_SIZE; i++) {
                    for(int j = 0; j < BoardData.BOARD_SIZE; j++) {
                        boardOutStream.writeShort(boardData.board[i][j]);
                    }
                }    
                
            } catch(IOException e) {
                System.out.println("Excpt write:" + e);
            }
            byte data[] = boardByteStream.toByteArray();            
            
            try {
                store.addRecord(data, 0, data.length);
            } catch (Exception e) {
                System.out.println("Excpt: addRecord()" + e);
            }
        }
        
    }
// low level loading data from the store to init the byte stream    
    private byte[] loadData(int record) {
        try { 
            if(store == null)
                openStore();
            return store.getRecord(record);
        } catch (Exception e) {
            System.out.println("Excpt: getRecord()" + e);
        }
        return null;
   }
// low level saving the variables to the store
    private void saveData(int record, byte[] data) {
        try { 
            if(store == null)
                openStore();          
            store.setRecord(record, data, 0, data.length);
        } catch (Exception e) {
            System.out.println("Excpt: setRecord()" + e);
        }
    }

}


