/*
 * MainCanvas.java
 * The main drawing class, uses the Board helper class
 *
 * v 1.0.1
 * some small bugs fixed: wsTimer() and sound in makeMove()
 *
 * v 1.0
 * first public release 
 *
 * (c) Arcanum
 * http://faqsite.narod.ru
 */

import javax.microedition.lcdui.*;
import java.io.*;
import java.util.Timer;
import java.util.TimerTask;
import com.siemens.mp.game.*;

public class MainCanvas extends Canvas implements CommandListener {
//define the images first
    static Image digImage;
    static Image letImage;
    static Image wsImage;
    static Image bsImage;
    static Image tsImage;
    
    static Image boardImage;    
    
    static Command quitSaveCommand;
    static Command quitCommand;
    static Command moveBackCommand;
    static Command backCommand;
    
    static final String STR_QUIT_SAVE   = "Quit and save";
    static final String STR_QUIT        = "Quit";
    static final String STR_BACK        = "Back";
    
    static {
        try {
            digImage    = Image.createImage("res/dig.png");  
            letImage    = Image.createImage("res/let.png");
            wsImage     = Image.createImage("res/ws.png");
            bsImage     = Image.createImage("res/bs.png");
            tsImage     = Image.createImage("res/ts.png");
            boardImage  = Image.createImage("res/bd.png");
        } catch(Exception e) {
            System.out.println("Excpt:" + e);
        }
//we'll implement some commands via CommandListener
        quitSaveCommand = new Command(STR_QUIT_SAVE, Command.EXIT, 0);
        quitCommand     = new Command(STR_QUIT, Command.EXIT, 1);
        backCommand     = new Command(STR_BACK, Command.BACK, 2);        
    }

    Displayable parent;   

    Board board;
    boolean sound;
    boolean light;
    int timeLimit;
    
    public MainCanvas(Displayable parent, int time, boolean sound, boolean light) {

        this.parent = parent;
        timeLimit   = time;
        this.sound  = sound;
        this.light  = light;       
        
        addCommand(quitCommand);
        addCommand(quitSaveCommand);
        addCommand(backCommand);
        this.setCommandListener(this);
        
        board = new Board(timeLimit);
    }
// for starting a new game    
    public void start(int time, boolean sound, boolean light)
    {
        timeLimit   = time;
        this.sound  = sound;
        this.light  = light;       
        
        board.start(timeLimit);
    }
// just resume the saved game
    public void resume() {
        board.resume();
    }
// all work for paintig is for Board class    
    protected void paint(Graphics g) {    
        board.drawBoard(g);
    }
// the key listener implements game control
    protected void keyPressed(int keyCode) {
    // translate keyCode to the suitable form
        int action = getGameAction(keyCode);
        switch(action) {
            case FIRE:
                board.makeMove();
                break;
            case LEFT:
                board.cursorLeft();
                break;
            case RIGHT:
                board.cursorRight();                
                break;
            case UP:
                board.cursorUp();                                
                break;
            case DOWN:
                board.cursorDown();                                
                break;              
        }
    // we have to repaint for changing game state
        repaint();
    } 
//here we'll work with he right soft key commands
    public void commandAction(Command cmd, Displayable dspl) {
        if(cmd == quitSaveCommand) {
            board.pause();            
            DataStore ds = new DataStore();
            ds.saveBoard(new BoardData(board.Field, board.move, board.bsTime, board.wsTime));
            Light.setLightOff();
            Display.getDisplay(RenjuBoard.midlet).setCurrent(parent);
        }
        if(cmd == quitCommand) {
            board.pause();
            Light.setLightOff();
            Display.getDisplay(RenjuBoard.midlet).setCurrent(parent);        
        }
    }
// The class for simulating renju board
    class Board {       
        static final int BOARD_SIZE     = 15;
        static final int SQUARE_SIZE    = 5;
    // the board offset from the top left corner
        static final int X_OFF          = 7;
        static final int Y_OFF          = 2;
    // the coordinates for the victoty message
        static final int VIC_X              = 17;
        static final int VIC_Y              = 25;        
    // color constants
        static final int GRID_COLOR         = 0x909090;
        static final int BOARD_COLOR        = 0xD8D800;
        static final int POINT_BORD_COLOR   = 0x909090;
        static final int POINT_CENT_COLOR   = 0x000000;
        static final int CURSOR_COLOR       = 0xFF0000;
        
    // the coordinats for state windows
        static final int IMG_STAT_X     = (BOARD_SIZE - 1) * SQUARE_SIZE + X_OFF + 3;
        static final int IMG_LAST_Y     = Y_OFF;
        static final int IMG_TURN_Y     = Y_OFF + 15;
        static final int IMG_TIME_Y     = Y_OFF + 30;        
    // digit's font constants
        static final int FONT_WIDTH     = 3;
        static final int FONT_HEIGHT    = 4;
        static final int LET_WIDTH      = 4;  
        static final int DIG_Y_OFF      = 1;
    // the board storage is the Field array
        static final int EMPTY_FIELD    = 0;
        static final int TMP_FIELD      = BOARD_SIZE * BOARD_SIZE + 2;
        int Field[][];
    // remember cursor position (0..BOARD_SIZE)
        int xCursor;
        int yCursor;
    //
        boolean blackMove = true;        
        int move;
        int xLastMove;
        int yLastMove;
    // the timers and the time
        Timer bsTimer;
        int bsTime;
        int wsTime;
        Timer wsTimer;
        
        boolean victory     = false;
        boolean pause       = false;
        int     fifthMove   = 0;
        
        public void pause() {
            pause = true;
        }
     // just show the victory after time out
        private void showVictory(Graphics g, boolean black) {
            if(sound)
                playEndSound();
            try {
                Image v = Image.createImage("res/vic.png");
                g.drawImage(v, VIC_X, VIC_Y, Graphics.LEFT | Graphics.TOP);
                drawStone(g, black, VIC_X + 6, VIC_Y + 12);
            } catch(Exception e) {
                System.out.println("Excpt:" + e);
            }
        }
    // black stones timer task. it runs every second
    // Some sophisticated checking for the rules of renju (the fifth move)        
        class BSTimerTask extends TimerTask {
            public void run() {
                if(victory || pause)
                    return;
                if(fifthMove == 2 && move == 5)
                    return;
                if(move < 3 || move == 5) {
                    bsTime--;
                    repaint();
                }
                else if(move % 2 == 0) {
                    if(bsTime == 0) {
                        victory = true;
                    }
                    else {
                        bsTime--;
                    }
                    repaint();
                }
            }
        }
    // white stones timer task. it runs every second
    // Some sophisticated checking for the rules of renju (the fifth move)        
        class WSTimerTask extends TimerTask {
            public void run() {
                if(victory || pause || move < 3)
                    return;                
                if(move % 2 != 0 || (fifthMove == 2 && move == 4)) {
                    if(wsTime == 0) {
                        victory = true;
                    }
                    else {    
                        wsTime--;
                        repaint();    
                    }
                }
            }
        }
        
        public Board(int time) {
            Field   = new int[BOARD_SIZE][BOARD_SIZE];
            xCursor = yCursor = BOARD_SIZE / 2;         
            bsTimer = new Timer();
            bsTimer.scheduleAtFixedRate(new BSTimerTask(), 1000, 1000); // 1 sec clock
            wsTimer = new Timer();
            wsTimer.scheduleAtFixedRate(new WSTimerTask(), 1000, 1000); // 1 sec clock           
            bsTime = time;
            wsTime = time;
        }
    // initial state of board        
        public void start(int time) {
            for(int i = 0; i < BOARD_SIZE; i++) {
                for(int j = 0; j < BOARD_SIZE; j++) {
                    Field[i][j] = EMPTY_FIELD;
                }
            }
            xCursor = yCursor = BOARD_SIZE / 2;         
            bsTime  = time;
            wsTime  = time;
            victory = false;
            pause   = false;
            fifthMove  = 0;
            move       = 0;
            
        }
        
    // resume saved game from the DataStore        
        public void resume() {
            DataStore ds = new DataStore();
            BoardData bd = ds.loadBoard();
            move    = bd.move;
            bsTime  = bd.bsTime;
            wsTime  = bd.wsTime;
            int maxMove = 0;
            for(int i = 0; i < BOARD_SIZE; i++) {
                for(int j = 0; j < BOARD_SIZE; j++) {
                    Field[i][j] = bd.board[i][j];
                    if(Field[i][j] > maxMove) {
                        xLastMove = i;
                        yLastMove = j;
                    }
                }
            }
            pause = false;
        }
    // just plays tones
        private void playMoveRightSound() {
            MelodyComposer melody = new MelodyComposer();
            try {
                melody.appendNote(MelodyComposer.TONE_C3, MelodyComposer.TONELENGTH_1_8);
                melody.appendNote(MelodyComposer.TONE_G3, MelodyComposer.TONELENGTH_1_8);
            } catch(Exception e) {
            }
            melody.setBPM(180);
            melody.getMelody().play();
        }
    // just plays tones        
        private void playMoveWrongSound() {
            MelodyComposer melody = new MelodyComposer();
            try {
                melody.appendNote(MelodyComposer.TONE_G2, MelodyComposer.TONELENGTH_1_8);
                melody.appendNote(MelodyComposer.TONE_C2, MelodyComposer.TONELENGTH_1_8);
            } catch(Exception e) {
            }
            melody.setBPM(180);
            melody.getMelody().play();
        }
   // just plays tones    
        private void playEndSound() {
            MelodyComposer melody = new MelodyComposer();
            try {
                melody.appendNote(MelodyComposer.TONE_E3, MelodyComposer.TONELENGTH_1_8);
                melody.appendNote(MelodyComposer.TONE_G3, MelodyComposer.TONELENGTH_1_8);
                melody.appendNote(MelodyComposer.TONE_H3, MelodyComposer.TONELENGTH_1_8);                
                melody.appendNote(MelodyComposer.TONE_C4, MelodyComposer.TONELENGTH_1_4);                                
            } catch(Exception e) {
            }
            melody.setBPM(180);
            melody.getMelody().play();
        }
        
  /* Print number with digit's font
   * x - right pos (right aligned), y - top pos
   */
        private void drawNumber(Graphics g, int number, int x, int y, boolean leadZero) {
            int n = number;
            for(int i = 1; ; i++) {
                int d = n - 10*(n/10);
                g.setClip(x -  FONT_WIDTH * i, y, FONT_WIDTH, FONT_HEIGHT);
                g.drawImage(digImage, x - i*FONT_WIDTH - d*FONT_WIDTH, y, Graphics.LEFT | Graphics.TOP);
                n /= 10;
                if(n == 0) {
                    if(i == 1 && leadZero) {
                        g.setClip(x -  FONT_WIDTH * 2, y, FONT_WIDTH, FONT_HEIGHT);
                        g.drawImage(digImage, x - 2*FONT_WIDTH, y, Graphics.LEFT | Graphics.TOP);
                    }                    
                    break;
                }
            }
            g.setClip(0,0,getWidth(), getHeight());
        }
    
    // just draw a rectangle
        private void drawCursor(Graphics g) {
            g.setColor(CURSOR_COLOR);
            g.drawRect(X_OFF + xCursor*SQUARE_SIZE - 3, Y_OFF + yCursor*SQUARE_SIZE - 3, 6, 6);
        }
           
        public void cursorLeft() {
            if(--xCursor < 0)
                xCursor = BOARD_SIZE - 1;
        }
        public void cursorRight() {
            if(++xCursor >= BOARD_SIZE)
                xCursor = 0;
        }
        public void cursorUp() {
            if(--yCursor < 0)
                yCursor = BOARD_SIZE - 1;
        }
        public void cursorDown() {
            if(++yCursor >= BOARD_SIZE)
                yCursor = 0;
        }       
        
        private void drawStone(Graphics g,  boolean black, int x, int y) {
            g.drawImage(black ? bsImage : wsImage, x, y, Graphics.HCENTER | Graphics.VCENTER);
        }
        
        private void drawTmpStone(Graphics g, int x, int y) {
            g.drawImage(tsImage, x, y, Graphics.HCENTER | Graphics.VCENTER);            
        }
    
    // draw the last move state
        private void drawMoveState(Graphics g) {
                if(move > 0) {
                    drawStone(g, move % 2 != 0, IMG_STAT_X + 5, IMG_LAST_Y + 8);
                    g.setClip(IMG_STAT_X + 9, IMG_LAST_Y + 7, SQUARE_SIZE, FONT_HEIGHT);
                    g.drawImage(letImage, IMG_STAT_X + 9 - xLastMove*SQUARE_SIZE, IMG_LAST_Y + 7, Graphics.LEFT | Graphics.TOP);
                    drawNumber(g, BOARD_SIZE - yLastMove, IMG_STAT_X + 19, IMG_LAST_Y + 7, false);
                }
        }
        
    // draw who's moving    
    // Some sophisticated checking for the rules of renju (the fifth move)                        
        private void drawTurnState(Graphics g) {
            boolean black = false;
            if(move < 3 || (move == 4 && fifthMove < 2)) {
                black = true;
            } else {               
                if((move == 4) && (fifthMove == 2))
                    black = false;
                else
                    black = move % 2 == 0;
            }
            drawNumber(g, move + 1, IMG_STAT_X + 11, IMG_TURN_Y + 7, false);
            drawStone(g, black, IMG_STAT_X + 14, IMG_TURN_Y + 8);
        }

    // draw who's moving time
    // Some sophisticated checking for the rules of renju (the fifth move)     
        private void drawTimeState(Graphics g) {
            int time = wsTime;
            if(move < 3 || (move == 4 && fifthMove < 2)) {
                time = bsTime;
            } 
            else {
                if(move == 4 && fifthMove == 2)
                    time = wsTime;
                else
                    time =  move % 2 != 0 ? wsTime : bsTime;
            }
            drawNumber(g, time / 60, IMG_STAT_X + 10, IMG_TIME_Y + 7, true);
            drawNumber(g, time % 60, IMG_STAT_X + 19, IMG_TIME_Y + 7, true); 
        }        
    
    // Fill the Field array with the last move
    // Some sophisticated checking for the rules of renju (the fifth move)             
        public void makeMove() {
            if(victory)
                return;
            if(Field[xCursor][yCursor] == EMPTY_FIELD) {
                if(move == 4) {
                    if(fifthMove < 2) {
                        Field[xCursor][yCursor] = TMP_FIELD;                    
                        fifthMove++;
                        if(sound)
                            playMoveRightSound();                        
                    }
                    else {
                        if(sound)
                            playMoveWrongSound();
                    }
                }
                else {
                    Field[xCursor][yCursor] = ++move;
                    xLastMove = xCursor;
                    yLastMove = yCursor;
                    if(sound)
                        playMoveRightSound();                    
                }
            }
            else {
                if(fifthMove == 2 && Field[xCursor][yCursor] == TMP_FIELD) {
                    Field[xCursor][yCursor] = ++move;
                    if(sound)
                        playMoveRightSound();                    
                    for(int i = 0; i < BOARD_SIZE; i++) {
                        for(int j = 0; j < BOARD_SIZE; j++) {
                            if(Field[i][j] == TMP_FIELD) {
                                Field[i][j] = EMPTY_FIELD;
                                break;
                            }
                        }                            
                    }
                }
                else if(sound)
                    playMoveWrongSound();
            }
        }
    
    // for rapainting
        public void drawBoard(Graphics g) {
        // the buffer image for drawing offscreen
            Image bufImage  = Image.createImage(getWidth(), getHeight());
            Graphics bg = bufImage.getGraphics();            
        // draw background board
            bg.drawImage(boardImage, 0, 0, Graphics.TOP | Graphics.LEFT);
        // draw stones
            for(int i = 0; i < BOARD_SIZE; i++) {
                for(int j = 0; j < BOARD_SIZE; j++) {
                    int field = Field[i][j];
                    if(field != EMPTY_FIELD) {
                        if(field == TMP_FIELD)
                            drawTmpStone(bg, X_OFF + SQUARE_SIZE * i, Y_OFF + SQUARE_SIZE * j);
                        else
                            drawStone(bg, field % 2 != 0, X_OFF + SQUARE_SIZE * i, Y_OFF + SQUARE_SIZE * j);
                    }
                }
            }
         // draw the state's values
            drawMoveState(bg);
            drawTurnState(bg);
            drawTimeState(bg);
         //   
         drawCursor(bg);
         //
         if(victory) {
             showVictory(bg, move % 2 != 0);
         }

        //finally put it on the screen (double buffering)
            g.drawImage(bufImage, 0, 0, Graphics.TOP | Graphics.LEFT);
            if(light)
                Light.setLightOn();
        }
    }    
}
