/*
 * SetList.java
 *
 * (c) Arcanum
 * http://faqsite.narod.ru
 */

import javax.microedition.lcdui.*;
import java.io.*;

public class SetList extends List implements CommandListener {
    Displayable parent;
    
    static Image tmeImage;
    static Image sndImage;
    static Image illImage;
    
    int         time;
    TimeForm    timeForm;
    boolean sound;
    boolean light;    
    
//Command constants
    static final int CMD_TIME           = 0;
    static final String STR_TIME        = "Time is";
    static final String STR_TIME_SET    = "Time setting in seconds";    
    static final String STR_SEC         = "s";    
    static final int CMD_SND            = 1; 
    static final String STR_SND_ON      = "Sound is on";
    static final String STR_SND_OFF     = "No sound";    
    static final int CMD_ILL            = 2;
    static final String STR_ILL_ON      = "Light is on";
    static final String STR_ILL_OFF     = "No Light";
    
    static final String STR_TIT         = "Settings";
    static final String STR_CMD_BACK    = "Back";
    static final String STR_CMD_CANCEL  = "Cancel";
    static final int MAX_TIME           = 4;
// the command to return to the parent
    static Command backCommand = new Command(STR_CMD_BACK, Command.BACK, 0);
// Preload the images
    static {
        try {
            tmeImage    = Image.createImage("res/tme.png");
            sndImage    = Image.createImage("res/snd.png");
            illImage    = Image.createImage("res/ill.png");
        }
        catch(IOException e) {
        }
    }
    
    private void refreshTime() {
        this.set(CMD_TIME, STR_TIME + " " + String.valueOf(time) + STR_SEC, tmeImage);
    }
    
    public SetList(Displayable parent) {
        super(STR_TIT, List.IMPLICIT);
        this.parent = parent;
        
        DataStore dataStore = new DataStore();
        SetData data        = dataStore.loadSettings();
        time    = data.time;
        sound   = data.sound;
        light   = data.light;        
      
        this.append(STR_TIME + " " + String.valueOf(time) + STR_SEC,  tmeImage);
        this.append(sound ? STR_SND_ON : STR_SND_OFF, sndImage);
        this.append(light ? STR_ILL_ON : STR_ILL_OFF, illImage);

        this.addCommand(backCommand);
        
        setCommandListener(this) ;
   }
    
    public void commandAction(Command cmd, Displayable dspl) {
        if(cmd == List.SELECT_COMMAND)
        {
        // the sample handler. Here it is empty. You can override this.
            int i = getSelectedIndex();
            switch(i)
            {
                case CMD_TIME:
                    if(timeForm == null)
                        timeForm = new TimeForm(this, time);
                    timeForm.show(time);
                    break;               
                case CMD_SND:
                    sound = !sound;
                    this.set(CMD_SND, sound ? STR_SND_ON : STR_SND_OFF, sndImage);
                    break;
                case CMD_ILL:
                    light = !light;
                    this.set(CMD_ILL, light ? STR_ILL_ON : STR_ILL_OFF, illImage);
                    break;        
                default:
                    break;
            }
        }
        if(cmd == backCommand)
        {
        // save all settings at the exit
            SetData data        = new SetData(time, sound, light);
            DataStore dataStore = new DataStore();
            dataStore.saveSettings(data);                
            Display.getDisplay(RenjuBoard.midlet).setCurrent(parent);
        }
    }
    
    class TimeForm extends Form implements CommandListener {
        SetList parent;
        TextField timeField;
        Command backCommand     = new Command(STR_CMD_BACK, Command.BACK, 0);
        Command cancelCommand   = new Command(STR_CMD_CANCEL, Command.CANCEL, 1);
       
        public TimeForm(SetList parent, int oldtime) {
            super(STR_TIME);            
            this.parent = parent;

            timeField = new TextField(STR_TIME_SET, "" + oldtime, MAX_TIME, TextField.NUMERIC);
            this.append(timeField);
            this.addCommand(backCommand);
            this.addCommand(cancelCommand);
            this.setCommandListener(this);
        }       
        
        public void show(int oldtime) {
            timeField.setString("" + oldtime);
            Display.getDisplay(RenjuBoard.midlet).setCurrent(this);
        }
        
        public void commandAction(Command cmd, Displayable dspl) {
            if(cmd == backCommand) {
                time = Integer.parseInt(timeField.getString());
                parent.refreshTime();                
                
                Display.getDisplay(RenjuBoard.midlet).setCurrent(parent);
            }
            if(cmd == cancelCommand) {
                Display.getDisplay(RenjuBoard.midlet).setCurrent(parent);                
            }
        }
        
    }
}
